@@ -90,6 +90,11 @@ class DXFile(DXDataObject):
     def set_http_threadpool_size(cls, num_threads):
         cls._http_threadpool_size = num_threads
 
+    @classmethod
+    def _ensure_http_threadpool(cls):
+        if cls._http_threadpool is None:
+            cls._http_threadpool = dxpy.utils.get_futures_threadpool(max_workers=cls._http_threadpool_size)
+
     def __init__(self, dxid=None, project=None, mode=None,
                  read_buffer_size=DEFAULT_BUFFER_SIZE, write_buffer_size=DEFAULT_BUFFER_SIZE):
         DXDataObject.__init__(self, dxid=dxid, project=project)
@@ -257,9 +262,22 @@ class DXFile(DXDataObject):
                 # offset is greater than original position but within the buffer
                 in_buf = True
 
-        if in_buf: # offset is within the buffer
+        if in_buf:
+            # offset is within the buffer (at least one byte following
+            # the offset can be read directly out of the buffer)
             self._read_buf.seek(orig_buf_pos - orig_pos + offset)
-        else: # offset is outside the buffer - reset buffer and queues. This is the failsafe behavior
+        elif offset == orig_pos:
+            # This seek is a no-op (the cursor is just past the end of
+            # the read buffer and coincides with the desired seek
+            # position). We don't have the data ready, but the request
+            # for the data starting here is already in flight.
+            #
+            # Detecting this case helps to optimize for sequential read
+            # access patterns.
+            pass
+        else:
+            # offset is outside the buffer-- reset buffer and queues.
+            # This is the failsafe behavior
             self._read_buf = BytesIO()
             # TODO: if the offset is within the next response(s), don't throw out the queues
             self._request_iterator, self._response_iterator = None, None
@@ -298,8 +316,7 @@ class DXFile(DXDataObject):
                 self._http_threadpool_futures = set()
 
     def _async_upload_part_request(self, *args, **kwargs):
-        if self._http_threadpool == None:
-            DXFile._http_threadpool = dxpy.utils.get_futures_threadpool(max_workers=self._http_threadpool_size)
+        self._ensure_http_threadpool()
 
         while len(self._http_threadpool_futures) >= self._http_threadpool_size:
             future = dxpy.utils.wait_for_a_future(self._http_threadpool_futures)
@@ -508,15 +525,17 @@ class DXFile(DXDataObject):
         if end_pos > self._file_length:
             raise DXFileError("Invalid end_pos")
 
-        def chunk_ranges(start_pos, end_pos, init_chunk_size=1024*64, limit_chunk_size=self._read_bufsize, ramp=4):
+        def chunk_ranges(start_pos, end_pos, init_chunk_size=1024*64, limit_chunk_size=self._read_bufsize, ramp=2, num_requests_between_ramp=4):
             cur_chunk_start = start_pos
             cur_chunk_size = min(init_chunk_size, limit_chunk_size)
+            i = 0
             while cur_chunk_start < end_pos:
                 cur_chunk_end = min(cur_chunk_start + cur_chunk_size - 1, end_pos)
                 yield cur_chunk_start, cur_chunk_end
                 cur_chunk_start += cur_chunk_size
-                if cur_chunk_size < limit_chunk_size:
+                if cur_chunk_size < limit_chunk_size and i % num_requests_between_ramp == (num_requests_between_ramp - 1):
                     cur_chunk_size = min(cur_chunk_size * ramp, limit_chunk_size)
+                i += 1
 
         for chunk_start_pos, chunk_end_pos in chunk_ranges(start_pos, end_pos):
             headers = copy.copy(headers)
@@ -530,12 +549,15 @@ class DXFile(DXDataObject):
                                                   'decode_response_body': False}
 
     def _next_response_content(self):
-        if self._http_threadpool is None:
-            DXFile._http_threadpool = dxpy.utils.get_futures_threadpool(max_workers=self._http_threadpool_size)
+        self._ensure_http_threadpool()
 
         if self._response_iterator is None:
-            self._response_iterator = dxpy.utils.response_iterator(self._request_iterator, self._http_threadpool,
-                                                                   max_active_tasks=self._http_threadpool_size)
+            self._response_iterator = dxpy.utils.response_iterator(
+                self._request_iterator,
+                self._http_threadpool,
+                max_active_tasks=self._http_threadpool_size,
+                queue_id=id(self)
+            )
         return next(self._response_iterator)
 
     def read(self, length=None, use_compression=None, **kwargs):
