@@ -16,7 +16,7 @@
 #   License for the specific language governing permissions and limitations
 #   under the License.
 
-from __future__ import print_function
+from __future__ import print_function, unicode_literals
 
 import logging
 logging.basicConfig(level=logging.WARNING)
@@ -37,6 +37,7 @@ from dxpy.utils.resolver import resolve_path, is_container_id
 from dxpy.app_categories import APP_CATEGORIES
 from dxpy.exceptions import err_exit, DXError
 from dxpy.utils.printing import BOLD
+from dxpy.compat import open, USING_PYTHON2
 
 parser = argparse.ArgumentParser(description="Uploads a DNAnexus App.")
 
@@ -254,10 +255,16 @@ def _lint(dxapp_json_filename):
         for i, input_field in enumerate(app_spec['inputSpec']):
             if not re.match("^[a-zA-Z_][0-9a-zA-Z_]*$", input_field['name']):
                 logger.error('input %d has illegal name "%s" (must match ^[a-zA-Z_][0-9a-zA-Z_]*$)' % (i, input_field['name']))
+    else:
+        logger.warn("dxapp.json contains no input specification (inputSpec). Your applet will not be usable as an " +
+                    "app, runnable from the GUI, or composable using workflows.")
     if 'outputSpec' in app_spec:
         for i, output_field in enumerate(app_spec['outputSpec']):
             if not re.match("^[a-zA-Z_][0-9a-zA-Z_]*$", output_field['name']):
                 logger.error('output %d has illegal name "%s" (must match ^[a-zA-Z_][0-9a-zA-Z_]*$)' % (i, output_field['name']))
+    else:
+        logger.warn("dxapp.json contains no output specification (outputSpec). Your applet will not be usable as an " +
+                    "app, runnable from the GUI, or composable using workflows.")
 
 def _check_syntax(code, lang, temp_dir, enforce=True):
     """
@@ -275,7 +282,7 @@ def _check_syntax(code, lang, temp_dir, enforce=True):
         raise ValueError('lang must be one of "python2.7" or "bash"')
     # Dump the contents out to a temporary file, then call _check_file_syntax.
     with open(os.path.join(temp_dir, temp_basename), 'w') as ofile:
-        ofile.write(code.encode('utf-8'))
+        ofile.write(code)
     _check_file_syntax(os.path.join(temp_dir, temp_basename), temp_dir, override_lang=lang, enforce=enforce)
 
 def _check_file_syntax(filename, temp_dir, override_lang=None, enforce=True):
@@ -295,6 +302,8 @@ def _check_file_syntax(filename, temp_dir, override_lang=None, enforce=True):
         # problems.
         pyc_path = os.path.join(temp_dir, os.path.basename(filename) + ".pyc")
         try:
+            if USING_PYTHON2:
+                filename = filename.encode(sys.getfilesystemencoding())
             py_compile.compile(filename, cfile=pyc_path, doraise=True)
         finally:
             try:
@@ -555,7 +564,7 @@ def _build_app_remote(mode, src_dir, publish=False, destination_override=None,
         # of binaries in it) and interrupt before uploading begins.
         app_tarball_file = os.path.join(temp_dir, "app_tarball.tar.gz")
         tar_subprocess = subprocess.Popen(["tar", "-czf", "-", "--exclude", "./.git", "."], cwd=src_dir, stdout=subprocess.PIPE)
-        with open(app_tarball_file, 'w') as tar_output_file:
+        with open(app_tarball_file, 'wb') as tar_output_file:
             total_num_bytes = 0
             last_console_update = 0
             start_time = time.time()
@@ -780,6 +789,8 @@ def main(**kwargs):
 
     if args.src_dir is None:
         args.src_dir = os.getcwd()
+        if USING_PYTHON2:
+            args.src_dir = args.src_dir.decode(sys.getfilesystemencoding())
 
     if args.mode == "app" and args.destination != '.':
         parser.error("--destination cannot be used when creating an app (only an applet)")
