@@ -23,42 +23,10 @@ from __future__ import (print_function, unicode_literals)
 import os, json, collections, concurrent.futures, traceback, time, gc
 import dateutil.parser
 from .exec_utils import run, convert_handlers_to_dxlinks, parse_args_as_job_input, entry_point
+from .thread_pool import PrioritizingThreadPool
 from .. import logger
 from ..compat import basestring
 
-# Monkeypatch ThreadPoolExecutor with relevant logic from the patch for
-# Python issue 16284. See:
-#
-#   <http://bugs.python.org/issue16284>
-#   <http://hg.python.org/cpython/rev/70cef0a160cf/>
-#
-# We may need to apply the relevant parts of the patches to
-# ProcessPoolExecutor and multiprocessing.Queue if we ever start using
-# those, too.
-def _non_leaky_worker(executor_reference, work_queue):
-    try:
-        while True:
-            try:
-                work_item = work_queue.get(block=True, timeout=0.1)
-            except concurrent.futures.thread.queue.Empty:
-                executor = executor_reference()
-                # Exit if:
-                #   - The interpreter is shutting down OR
-                #   - The executor that owns the worker has been collected OR
-                #   - The executor that owns the worker has been shutdown.
-                if concurrent.futures.thread._shutdown or executor is None or executor._shutdown:
-                    return
-                del executor
-            else:
-                work_item.run()
-                del work_item # <= free this item before the next
-                              #    work_queue.get call runs, rather than
-                              #    after
-    except BaseException:
-        concurrent.futures.thread._base.LOGGER.critical('Exception in worker', exc_info=True)
-
-concurrent.futures.thread._worker = _non_leaky_worker
-
 
 def _force_quit(signum, frame):
     # traceback.print_stack(frame)
@@ -66,9 +34,11 @@ def _force_quit(signum, frame):
     # os.abort()
 
 def get_futures_threadpool(max_workers):
+    #import signal
     #if force_quit_on_sigint:
     #    signal.signal(signal.SIGINT, _force_quit)
-    return concurrent.futures.ThreadPoolExecutor(max_workers=max_workers)
+    #return concurrent.futures.ThreadPoolExecutor(max_workers=max_workers)
+    return PrioritizingThreadPool(max_workers=max_workers)
 
 def wait_for_a_future(futures, print_traceback=False):
     '''
@@ -117,13 +87,20 @@ def wait_for_all_futures(futures, print_traceback=False):
             print('')
         os._exit(os.EX_IOERR)
 
-def response_iterator(request_iterator, worker_pool, max_active_tasks=4, num_retries=0, retry_after=90):
+def response_iterator(request_iterator, thread_pool, max_active_tasks=4, num_retries=0, retry_after=90, queue_id=''):
     '''
     :param request_iterator: This is expected to be an iterator producing inputs for consumption by the worker pool.
-    :param worker_pool: Assumed to be a concurrent.futures.Executor instance.
+    :type request_iterator: iterator of callable_, args, kwargs
+    :param thread_pool: thread pool to submit the requests to
+    :type thread_pool: PrioritizingThreadPool
     :param max_active_tasks: The maximum number of tasks that may be either running or waiting for consumption of their result.
+    :type max_active_tasks: int
     :param num_retries: The number of times to retry the request.
+    :type num_retries: int
     :param retry_after: The number of seconds to wait before retrying the request.
+    :type retry_after: number
+    :param queue_id: hashable object to divide incoming requests into independent queues
+    :type queue_id: object
 
     Rate-limited asynchronous multithreaded task runner.
     Consumes tasks from *request_iterator*. Yields their results in order, while allowing up to *max_active_tasks* to run
@@ -141,56 +118,84 @@ def response_iterator(request_iterator, worker_pool, max_active_tasks=4, num_ret
     #    yield _callable(*args, **kwargs)
     #return
 
-    def submit(pool, _callable, args, kwargs, retries=num_retries):
-        # print "Submitting", _callable, args, kwargs
-        future = pool.submit(_callable, *args, **kwargs)
-        future.args = (_callable, args, kwargs)
-        future.retries = retries
-        return future
+    num_results_yielded = 0
+    next_request_index = 0
 
-    def resubmit(pool, future):
-        _callable, args, kwargs = future.args
-        logger.warn("{}: Retrying {} after timeout".format(__name__, _callable))
-        return submit(pool, _callable, args, kwargs, future.retries-1)
+    def make_priority_fn(request_index):
+        # The more pending requests are between the data that has been
+        # returned to the caller and this data, the less likely this
+        # data is to be needed soon. This results in a higher number
+        # here (and therefore a lower priority).
+        return lambda: request_index - num_results_yielded
 
-    future_deque = collections.deque()
-    for i in range(max_active_tasks):
+    def submit(callable_, args, kwargs, retries=num_retries):
+        """Submit the task.
+
+        Return (future, (callable_, args, kwargs), retries)
+
+        """
+        future = thread_pool.submit_to_queue(queue_id, make_priority_fn(next_request_index), callable_, *args, **kwargs)
+        return (future, (callable_, args, kwargs), retries)
+
+    def resubmit(callable_, args, kwargs, retries):
+        """Submit the task.
+
+        Return (future, (callable_, args, kwargs), retries)
+
+        """
+        logger.warn("{}: Retrying {} after timeout".format(__name__, callable_))
+        # TODO: resubmitted tasks should be prioritized higher
+        return submit(callable_, args, kwargs, retries=retries-1)
+
+    # Each item is (future, (callable_, args, kwargs), retries):
+    #
+    # future: Future for the task being performed
+    # callable_, args, kwargs: callable and args that were supplied
+    # retries: number of additional times they request may be retried
+    tasks_in_progress = collections.deque()
+
+    for _i in range(max_active_tasks):
         try:
-            _callable, args, kwargs = next(request_iterator)
-            # print "Submitting (initial batch):", _callable, args, kwargs
-            f = submit(worker_pool, _callable, args, kwargs)
-            future_deque.append(f)
+            callable_, args, kwargs = next(request_iterator)
+            # print "Submitting (initial batch):", callable_, args, kwargs
+            tasks_in_progress.append(submit(callable_, args, kwargs))
+            next_request_index += 1
         except StopIteration:
             break
 
-    while len(future_deque) > 0:
+    while len(tasks_in_progress) > 0:
+        future, callable_and_args, retries = tasks_in_progress.popleft()
         try:
-            f = future_deque.popleft()
-            result = f.result(timeout=retry_after)
+            result = future.result(timeout=retry_after)
         except concurrent.futures.TimeoutError:
             # print "Timeout while waiting for", f, "which has", f.retries, "retries left"
-            if f.retries > 0 and len(future_deque) > 2 and all(f.done() for f in future_deque):
+            if retries > 0 and len(tasks_in_progress) > 2 and all(f.done() for (f, _callable, _retries) in tasks_in_progress):
                 # The stale future will continue to run and will reduce the effective size of the pool by 1. If too many
                 # futures are retried, the pool will block until one of the stale futures quits.
                 # f.cancel() doesn't work because there's no way to interrupt a thread.
-                f = resubmit(worker_pool, f)
-            future_deque.appendleft(f)
+                prev_callable, prev_args, prev_kwargs = callable_and_args
+                future, callable_and_args, retries = resubmit(prev_callable, prev_args, prev_kwargs, retries)
+                next_request_index += 1
+            tasks_in_progress.appendleft((future, callable_and_args, retries))
             continue
         except KeyboardInterrupt:
             print('')
             os._exit(os.EX_IOERR)
 
-        del f # Free the future we just consumed now, instead of next
-              # time around the loop
+        del future # Free the future we just consumed now, instead of next
+                   # time around the loop
         gc.collect()
 
         try:
-            _callable, args, kwargs = next(request_iterator)
-            next_future = submit(worker_pool, _callable, args, kwargs)
-            future_deque.append(next_future)
+            callable_, args, kwargs = next(request_iterator)
         except StopIteration:
             pass
+        else:
+            tasks_in_progress.append(submit(callable_, args, kwargs))
+            next_request_index += 1
         yield result
+        del result
+        num_results_yielded += 1
 
 def string_buffer_length(buf):
     orig_pos = buf.tell()
